@@ -1253,13 +1253,8 @@ ev_print_operation_export_print_dialog_response_cb (GtkDialog              *dial
 
 	get_first_and_last_page (export, &first_page, &last_page);
 
-	if (file_format) {
-		export->fc.format = g_ascii_strcasecmp (file_format, "pdf") == 0 ?
-			EV_FILE_FORMAT_PDF : EV_FILE_FORMAT_PS;
-	} else {
-		export->fc.format = gtk_printer_accepts_pdf (printer) ?
-			EV_FILE_FORMAT_PDF : EV_FILE_FORMAT_PS;
-	}
+	export->fc.format = file_format && g_ascii_strcasecmp (file_format, "pdf") == 0 ?
+		EV_FILE_FORMAT_PDF : EV_FILE_FORMAT_PS;
 	export->fc.filename = export->temp_file;
 	export->fc.first_page = MIN (first_page, last_page);
 	export->fc.last_page = MAX (first_page, last_page);
@@ -1642,6 +1637,24 @@ print_job_cancelled (EvJobPrint            *job,
 	gtk_print_operation_cancel (print->op);
 }
 
+static void
+ev_print_operation_print_request_page_setup (EvPrintOperationPrint *print,
+					     GtkPrintContext       *context,
+					     gint                   page_nr,
+					     GtkPageSetup          *setup)
+{
+	EvPrintOperation *op = EV_PRINT_OPERATION (print);
+	gint              width, height;
+
+	ev_page_cache_get_size (ev_page_cache_get (op->document),
+				page_nr, 0, 1.0,
+				&width, &height);
+	if (width > height)
+	        gtk_page_setup_set_orientation (setup, GTK_PAGE_ORIENTATION_LANDSCAPE);
+	else
+	        gtk_page_setup_set_orientation (setup, GTK_PAGE_ORIENTATION_PORTRAIT);
+}
+
 static void
 ev_print_operation_print_draw_page (EvPrintOperationPrint *print,
 				    GtkPrintContext       *context,
@@ -1651,6 +1664,7 @@ ev_print_operation_print_draw_page (EvPrintOperationPrint *print,
 	cairo_t          *cr;
 	gdouble           cr_width, cr_height;
 	gint              width, height;
+	gdouble           scale;
 
 	gtk_print_operation_set_defer_drawing (print->op);
 
@@ -1672,7 +1686,11 @@ ev_print_operation_print_draw_page (EvPrintOperationPrint *print,
 	ev_page_cache_get_size (ev_page_cache_get (op->document),
 				page, 0, 1.0,
 				&width, &height);
-	cairo_scale (cr, cr_width / (gdouble)width, cr_height / (gdouble)height);
+	if (cr_width / (gdouble)width < cr_height / (gdouble)height)
+	        scale = cr_width / (gdouble)width;
+	else
+	        scale = cr_height / (gdouble)height;
+	cairo_scale (cr, scale, scale);
 
 	ev_job_print_set_cairo (EV_JOB_PRINT (print->job_print), cr);
 	ev_job_scheduler_push_job (print->job_print, EV_JOB_PRIORITY_NONE);
@@ -1725,6 +1743,9 @@ ev_print_operation_print_init (EvPrintOperationPrint *print)
 	g_signal_connect_swapped (print->op, "status_changed",
 				  G_CALLBACK (ev_print_operation_print_status_changed),
 				  print);
+	g_signal_connect_swapped (print->op, "request_page_setup",
+				  G_CALLBACK (ev_print_operation_print_request_page_setup),
+				  print);
 	gtk_print_operation_set_allow_async (print->op, TRUE);
 }
 
